@@ -157,13 +157,13 @@ gst_mpeg2dec_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_template_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template_factory));
+  gst_element_class_add_static_pad_template (element_class,
+      &src_template_factory);
+  gst_element_class_add_static_pad_template (element_class,
+      &sink_template_factory);
 #ifdef enable_user_data
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&user_data_template_factory));
+  gst_element_class_add_static_pad_template (element_class,
+      &user_data_template_factory);
 #endif
   gst_element_class_set_details_simple (element_class,
       "mpeg1 and mpeg2 video decoder", "Codec/Decoder/Video",
@@ -284,6 +284,8 @@ gst_mpeg2dec_qos_reset (GstMpeg2dec * mpeg2dec)
   GST_OBJECT_LOCK (mpeg2dec);
   mpeg2dec->proportion = 1.0;
   mpeg2dec->earliest_time = -1;
+  mpeg2dec->dropped = 0;
+  mpeg2dec->processed = 0;
   GST_OBJECT_UNLOCK (mpeg2dec);
 }
 
@@ -918,10 +920,33 @@ handle_slice (GstMpeg2dec * mpeg2dec, const mpeg2_info_t * info)
         && qostime <= mpeg2dec->earliest_time;
     GST_OBJECT_UNLOCK (mpeg2dec);
 
-    if (need_skip)
+    if (need_skip) {
+      GstMessage *qos_msg;
+      guint64 stream_time;
+      gint64 jitter;
+
+      mpeg2dec->dropped++;
+
+      stream_time =
+          gst_segment_to_stream_time (&mpeg2dec->segment, GST_FORMAT_TIME,
+          time);
+      jitter = GST_CLOCK_DIFF (qostime, mpeg2dec->earliest_time);
+
+      qos_msg =
+          gst_message_new_qos (GST_OBJECT_CAST (mpeg2dec), FALSE, qostime,
+          stream_time, time, GST_BUFFER_DURATION (outbuf));
+      gst_message_set_qos_values (qos_msg, jitter, mpeg2dec->proportion,
+          1000000);
+      gst_message_set_qos_stats (qos_msg, GST_FORMAT_BUFFERS,
+          mpeg2dec->processed, mpeg2dec->dropped);
+      gst_element_post_message (GST_ELEMENT_CAST (mpeg2dec), qos_msg);
+
       goto dropping_qos;
+    }
   }
 
+  mpeg2dec->processed++;
+
   /* ref before pushing it out, so we still have the ref in our
    * array of buffers */
   gst_buffer_ref (outbuf);
@@ -1150,7 +1175,7 @@ gst_mpeg2dec_chain (GstPad * pad, GstBuffer * buf)
               ("%d consecutive decoding errors", mpeg2dec->error_count),
               (NULL));
         }
-        goto exit;
+        continue;
       default:
         GST_ERROR_OBJECT (mpeg2dec, "Unknown libmpeg2 state %d, FIXME", state);
         goto exit;
