@@ -105,10 +105,9 @@ gst_dvd_sub_dec_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&subtitle_template));
+  gst_element_class_add_static_pad_template (element_class, &src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &subtitle_template);
 
   gst_element_class_set_details_simple (element_class, "DVD subtitle decoder",
       "Codec/Decoder/Video", "Decodes DVD subtitles into AYUV video frames",
